<?php
// version 2.0.1
use WHMCS\Database\Capsule;

include_once 'PapApi.class.php'; // -------------------------------------------- Change to correct path
add_hook('AfterShoppingCartCheckout', 20, 'pap_Track', '');
add_hook('InvoicePaid', 0, 'pap_InvoicePaid', '');
add_hook('InvoiceUnpaid', 0, 'pap_InvoiceDenied', '');
add_hook('InvoiceCancelled', 0, 'pap_InvoiceDenied', '');
add_hook('InvoiceRefunded', 0, 'pap_InvoiceRefunded', '');
// add_hook('ClientAdd',20,'pap_AddAffiliate', ''); // ------------------------- Uncomment if you want to track affiliates as well

class WHMCSPap {

    /* ******************** CONFIGURATION PART ****************************** */
    /* ********************** MERCHANT DETILS ******************************* */
    protected $pap_url = 'http://youraccount.postaffiliatepro.com/'; // -------- or http://yoursite.com/pap4_path/
    protected $pap_user = 'email@yourdomain.com'; // --------------------------- Username
    protected $pap_password = 'yourpass'; // ----------------------------------- Password

    /* ********************** TRACKING DETILS ******************************* */
    protected $debug = false;
    protected $logfile = 'includes/hooks/pap.log'; // -------------------------- Path to log file
    protected $trackOrdersBy = 'type'; // 'type' (= "Hosting", "Addon", "Setup"), 'productId'
    /* ********************************************************************** */
    /* ******************** /CONFIGURATION PART ***************************** */

    protected function pap_log($message) {
        $log = fopen($this->logfile, 'a');
        fwrite($log, date('Y-m-d H:i:s') . ' : ' . $message . "\n");
        fclose($log);
    }

    public function track($vars) {
        if ($this->debug) {
            $this->pap_log('Tracking order');
        }

        $invoiceid = $vars['invoiceid'];
        if (!$invoiceid) {
            $this->trackFreeProduct($vars);
            return;
        }

        $sql = 'SELECT i.userid,i.description,i.amount,inv.status,o.promocode,';

        if ($this->trackOrdersBy == 'productId') {
            $sql .= "h.packageid as productId FROM tblinvoiceitems as i,tblinvoices as inv,tblorders as o,tblhosting as h WHERE i.invoiceid='$invoiceid' AND inv.id = i.invoiceid AND o.invoiceid = i.invoiceid AND i.relid=h.id";
        } else {
            $sql .= "i.type as productId FROM tblinvoiceitems as i,tblinvoices as inv,tblorders as o WHERE i.invoiceid='$invoiceid' AND inv.id = i.invoiceid AND o.invoiceid = i.invoiceid";
        }

        $total_query = Capsule::select(Capsule::raw($sql));

        if ($this->debug) {
            $this->pap_log("Creating sale for $invoiceid at URL: " . $this->pap_url);
        }

        $i = 1;
        foreach ($total_query as $row) {
            $saleTracker = new Pap_Api_SaleTracker($this->pap_url . 'scripts/sale.php');
            $sale = $saleTracker->createSale();
            $sale->setTotalCost($row->amount);
            $sale->setOrderID("$invoiceid($i)");
            $sale->setProductId($row->productId);
            $sale->setData1($row->userid);
            $sale->setData2($row->description);
            $sale->setStatus($this->resolveStatus($row->status));
            $sale->setCouponCode($row->promocode);
            $saleTracker->register();

            if ($this->debug)
                $this->pap_log("Order $invoiceid($i) tracked");
            $i++;
        }

        return;
    }
    
    private function trackFreeProduct($vars) {
		$this->pap_log('Submitting a commission for free product based on order with ID '.$vars['orderid'].' and order # '.$vars['ordernumber']);

		foreach ($vars['serviceids'] as $hostingId) {
			try {
				$saleTracker = new Pap_Api_SaleTracker($this->pap_url.'scripts/sale.php');
				$sale = $saleTracker->createSale();
				$sale->setTotalCost($vars['totaldue']);
				$sale->setOrderID($hostingId);
				$hosting = Capsule::table('tblhosting')->where('id', $hostingId)->first();
				$sale->setProductID($hosting->packageid);
				$sale->setData1($hosting->userid);
				$saleTracker->register();
				
				$this->pap_log('Commision submitted for saving with order ID '.$hostingId.'.');
			} catch (Exception $e) {
				$this->pap_log('Error during commission submision : '.$e->getMessage());
			}
		}
	}

    private function resolveStatus($dbStatus) {
        switch ($dbStatus) {
            case 'Paid':
                return 'A';
            case 'Cancelled':
            case 'Refunded':
                return 'D';
            default:
                return 'P';
        }
    }

    public function refund($vars) {
        $orderid = $vars['invoiceid'];
        $session = new Gpf_Api_Session($this->pap_url . 'scripts/server.php');
        if (!@$session->login($this->pap_user, $this->pap_password)) {
            if ($this->debug)
                $this->pap_log('Cannot login. Message: ' . $session->getMessage());
            return;
        }

        $request = new Pap_Api_TransactionsGrid($session);
        $request->addFilter('orderId', Gpf_Data_Filter::LIKE, $orderid . '\(%'); // find all transactions (123(1), 123(2), ...)
        $request->setSorting('orderid', false);
        $request->sendNow();
        $grid = $request->getGrid();

        if ($grid->getTotalCount() > 0) {
            $recordset = $grid->getRecordset();
            foreach ($recordset as $rec) {
                $this->refundRec($rec, $session);
            }

            $totalRecords = $grid->getTotalCount();

            $maxRecords = $recordset->getSize();

            if ($maxRecords > 0) {
                $cycles = ceil($totalRecords / $maxRecords);
                for($i = 1; $i < $cycles; $i++) {
                    $request->setLimit($i * $maxRecords, $maxRecords);
                    $request->sendNow();
                    $recordset = $request->getGrid()->getRecordset();

                    foreach ($recordset as $rec) {
                        $this->refundRec($rec, $session);
                    }
                }
            }
        }
    }

    protected function refundRec($rec, $session) {
        $sale = new Pap_Api_Transaction($session);
        $sale->setTransid($rec->get('id'));
        try {
            if (!$sale->load()) {
                if ($this->debug)
                    $this->pap_log('Cannot load transaction for' . $rec->get('id') . ' error: ' . $sale->getMessage());
            } else {
                $sale->refund();
                if ($this->debug)
                    $this->pap_log('Transaction ' . $rec->get('id') . ' refunded');
            }
        } catch (Exception $e) {
            if ($this->debug)
                $this->pap_log('Exception: ' . $e->getMessage());
            return;
        }
    }

    public function changePAP4status($invoiceid, $approved) {
        if (empty($invoiceid)) {
            if ($this->debug)
                $this->pap_log('Invoice empty! Ending.');
            return;
        }
        if ($this->debug)
            $this->pap_log("Changing PAP4 status of invoice $invoiceid to status: $approved");

        $pap_query = Capsule::table('tblorders')
                ->select('id')
                ->where('invoiceid', '=', $invoiceid)
                ->get();

        if (empty($pap_query)) {
            if ($approved) {
                if ($this->debug)
                    $this->pap_log('This seems to be a recurring order...');
                $this->recurringCommission($invoiceid);
                return;
            }
        }

        if ($this->debug)
            $this->pap_log("Changing status for invoice $invoiceid");

        $session = new Gpf_Api_Session($this->pap_url . 'scripts/server.php');
        if (!@$session->login($this->pap_user, $this->pap_password)) {
            if ($this->debug)
                $this->pap_log('Cannot login. Message: ' . $session->getMessage());
            return;
        }

        // Get all orders with the same order id
        $request = new Pap_Api_TransactionsGrid($session);
        $request->addFilter('orderId', Gpf_Data_Filter::LIKE, $invoiceid . '\(%');
        $request->sendNow();
        $grid = $request->getGrid();
        $recordset = $grid->getRecordset();

        if ($grid->getTotalCount() == 0) {
            if ($this->debug)
                $this->pap_log('No transactions found, let\'s try unprocessed transactions now.');

            $transaction = new Pap_Api_Transaction($session);
            if ($approved) { // for approval
                if ($this->debug)
                    $this->pap_log('Order ' . $invoiceid . ' will be approved.');
                for($i = 0; $i < 30; $i++) {
                    $transaction->setOrderId("$invoiceid($i)");
                    $transaction->approveByOrderId();
                }
            } else { // for declining
                if ($this->debug)
                    $this->pap_log('Order ' . $invoiceid . ' will be declined.');
                for($i = 0; $i < 30; $i++) {
                    $transaction->setOrderId("$invoiceid($i)");
                    $transaction->declineByOrderId();
                }
            }
            return;
        }

        $ids = array();
        foreach ($recordset as $rec) {
            $ids[] = $rec->get('id');
        }

        $totalRecords = $grid->getTotalCount();

        $maxRecords = $recordset->getSize();

        if ($maxRecords > 0) {
            $cycles = ceil($totalRecords / $maxRecords);
            for($i = 1; $i < $cycles; $i++) {
                $request->setLimit($i * $maxRecords, $maxRecords);
                $request->sendNow();
                $recordset = $request->getGrid()->getRecordset();

                foreach ($recordset as $rec) {
                    $ids[] = $rec->get('id');
                }
            }
        }

        $status = 'D';
        if ($approved) {
            $status = 'A';
        }

        try {
            if ($this->debug)
                $this->pap_log('IDs to change status for: ' . print_r($ids, true));
            $request = new Gpf_Rpc_FormRequest('Pap_Merchants_Transaction_TransactionsForm', 'changeStatus', $session);
            $request->addParam('ids', new Gpf_Rpc_Array($ids));
            $request->addParam('status', $status);
            $request->sendNow();
            if ($this->debug)
                $this->pap_log('IDs status change successful.');
        } catch (Exception $e) {
            if ($this->debug)
                $this->pap_log('API error while status changing: ' . $e->getMessage());
            return false;
        }
    }

    public function recurringCommission($invoiceid) {
        if ($this->debug) {
            $this->pap_log("Starting tracking a recurring order $invoiceid...");
        }

        $sql = 'SELECT i.userid,i.description,i.amount,inv.status,o.promocode,i.type as type';

        if ($this->trackOrdersBy == 'productId') {
            $sql .= "h.packageid as productId FROM tblinvoiceitems as i,tblinvoices as inv,tblorders as o,tblhosting as h WHERE i.invoiceid='$invoiceid' AND inv.id = i.invoiceid AND o.invoiceid = i.invoiceid AND i.relid=h.id";
        } else {
            $sql .= " FROM tblinvoiceitems as i,tblinvoices as inv,tblorders as o WHERE i.invoiceid='$invoiceid' AND inv.id = i.invoiceid AND o.invoiceid = i.invoiceid";
        }


        $i = 1;
        foreach ($pap_query as $row) {
            if ($row->type == 'Invoice') { // mass payment, no new commission, we have to approve existing one
                $oldInvoiceId = substr($row->description, strpos($row->description, '#') + 1); // e.g. Invoice #1234
                $this->changePAP4status($oldInvoiceId, 1);
                continue;
            }

            $saleTracker = new Pap_Api_SaleTracker($this->pap_url . 'scripts/sale.php');
            $sale = $saleTracker->createSale();
            $sale->setTotalCost($row->amount);
            $sale->setOrderID("$invoiceid($i)");
            if ($this->trackOrdersBy == 'productId') {
                $sale->setProductId($row->productId);
            } else {
                $sale->setProductId($row->type);
            }
            $sale->setData1($row->userid);
            $sale->setData2($row->description);
            $sale->setStatus($this->resolveStatus($row->status));
            $sale->setCouponCode($row->promocode);
            $saleTracker->register();
            $i++;
            if ($this->debug)
                $this->pap_log("Order $invoiceid tracked");
        }
        return;
    }

    public function addAffiliate($vars) {
		if ($this->debug) {
			$this->pap_log('Adding affiliate with data: '.print_r($vars,true));
		}
		$session = new Gpf_Api_Session($this->pap_url . 'scripts/server.php');
		if (!$session->login($this->pap_user, $this->pap_password)) {
			if($this->debug) {
				$this->pap_log('Cannot login. Message: '.$session->getMessage());
			}
			return;
		}

		$affiliate = new Pap_Api_AffiliateSignup($session);
		$affiliate->setUsername($vars['email']);
		$affiliate->setNotificationEmail($vars['email']);
		$affiliate->setFirstname($vars['firstname']);
		$affiliate->setLastname($vars['lastname']);
		$affiliate->setPassword($vars['password']);
		$affiliate->setData(1, $vars['userid']);
		$affiliate->setData(2, $vars['companyname']);
		$affiliate->setData(3, $vars['address1']);
		$affiliate->setData(4, $vars['city']);
		$affiliate->setData(5, $vars['state']);
		$affiliate->setData(6, $vars['country']);
		$affiliate->setData(7, $vars['postcode']);
		$affiliate->setData(8, $vars['phonenumber']);
		$affiliate->setData(25, $vars['userid']); //recording the WHMCS userid

		try {
			if ($affiliate->add()) {
                if ($this->debug)
                $this->pap_log('Affiliate successfully added');
			} else {
			  if ($this->debug) {
				$this->pap_log('Cannot save affiliate: '.$affiliate->getMessage());
			}

			}
		} catch (Exception $e) {
			if ($this->debug) {
				$this->pap_log('Error while communicating with PAP: '.$e->getMessage());
			}
		}
		return;
	}
}

function pap_InvoiceDenied($vars) {
    // marking the transaction as declined in PAP
    $vars = array_change_key_case($vars);
    $pap = new WHMCSPap();
    $pap->changePAP4status($vars['invoiceid'], 0);
}

function pap_InvoicePaid($vars) {
    // marking the transaction as approved in PAP
    $vars = array_change_key_case($vars);
    $pap = new WHMCSPap();
    $pap->changePAP4status($vars['invoiceid'], 1);
}

function pap_InvoiceRefunded($vars) {
    // marking the transaction as refunded in PAP
    $vars = array_change_key_case($vars);
    $pap = new WHMCSPap();
    $pap->refund($vars);
}

function pap_Track($vars) {
    // tracking a new transaction in PAP
    $vars = array_change_key_case($vars);
    $pap = new WHMCSPap();
    $pap->track($vars);
}

function pap_AddAffiliate($vars) {
	$pap = new WHMCSPap();
	$pap->addAffiliate($vars);
}
